// (C) 1997, 1998 by Paolo Gatti
//
// $Id: GridView.h,v 1.7 1998/11/01 21:45:18 paolo Exp $
// $Log: GridView.h,v $
// Revision 1.7  1998/11/01 21:45:18  paolo
// R4 compatibility
//
// Revision 1.6  1998/01/05 20:59:49  paolo
// Implemented LockSep and NewChild, stricter typing, first public release
//
// Revision 1.5  1998/01/03 22:17:57  paolo
// Added archival/instantiation
//
// Revision 1.3  1997/12/25 12:43:51  paolo
// Implemented Add/RemoveChild, ChildAt, subview resizing, etc.
// First functioning version
//
// Revision 1.2  1997/12/22 10:17:12  paolo
// Complete revision
//
// Revision 1.1  1997/12/14 20:24:48  paolo
// Initial revision

#ifndef __GRIDVIEW_H
#define __GRIDVIEW_H

#define __GRIDVIEW__ "GridView"

#ifndef __ARRAY2D_H
#include "Array2d.h"
#endif  /* __ARRAY2D_H */

class GridView : public BView
{
public:
	GridView(	BRect frame,
				const char *name,
				uint32 resizingMode,
				uint32 flags,
				float bordersize,
				float minseparation);
	GridView(BMessage *archive);
	virtual ~GridView();
	static GridView *Instantiate(BMessage *msg);
	virtual status_t Archive(BMessage *archive, bool deep) const;
	
	virtual void Draw(BRect updateRect);
//	virtual void FrameResized(float width, float height);
	virtual void MouseDown(BPoint point);

	virtual void NewChild(int32 id);
	virtual void DrawEmptyChild(BRect rect, BRect updateRect);
	virtual void ClickOnChild(BPoint point, int32 x, int32 y);
	virtual void ChildDetached(BView *view);

	void Lock();
	void Unlock();
	void LockSep();
	void UnlockSep();

	void DragSeparator(int32 idx[2], float coords[2], uint32 buttons);
	void DragSeparator(int32 x, int32 y, float cx, float cy, uint32 buttons);

	void AddChild(BView *aView, int32 id);
	void AddChild(BView *aView, int32 x, int32 y);
	bool RemoveChild(BView *aView);
	bool RemoveChild(int32 id);
	bool RemoveChild(int32 x, int32 y);

	int32 Id(int32 x, int32 y) const;
	int32 Id(BPoint where) const;
	BView *ChildAt(int32 id) const;
	BView *ChildAt(int32 x, int32 y) const;

	int32 CountSeparators(int32 dir) const;
	float SeparatorAt(int32 dir, int32 index) const;

private:
	void AddChild(BView *aView, BView *sibling = NULL);
	void DrawCursorX(float where);
	void DrawCursorY(float where);
	void GetCoords(float coords[2], int32 idx[2], float minsep) const;
	void AlignCoords(int32 idx[2]) const;
	int32 CalcSgmt(int32 idx[2], int32 sgmt[2]) const;
	void CalcToMov(int32 mode, int32 sgmt[2], int32 idx[2], int32 mint32omov[2],
					int32 maxtomov[2], uint32 buttons) const;
	void CalcBounds(int32 mode, int32 idx[2], int32 mint32omov[2], int32 maxtomov[2],
					float xcoords[2]) const;
	void CommitChanges(int32 mode, int32 idx[2], int32 newidx[2],
						int32 mint32omov[2], int32 maxtomov[2], float coords[2]);
	void RemoveEmptySeparator(int32 mode, int32 which, int32 howmany[2]);

	BList separators[2];
	BList children;
	Array2d<int32> *views;
	float bordersize;
	float minseparation;
	int32 locked;
	int32 seplocked;
	int32 subviewnum;
};

inline void GridView::Lock()
{
	locked++;
}

inline void GridView::Unlock()
{
	if (locked)
		locked--;
}

inline void GridView::LockSep()
{
	seplocked++;
}

inline void GridView::UnlockSep()
{
	if (seplocked) {
		if (!(--seplocked)) {

			int32 n, howmany[2];
			howmany[0] = separators[0].CountItems();
			howmany[1] = separators[1].CountItems();
			for (n = howmany[0] - 2; n > 0; n--)
				RemoveEmptySeparator(0, n, howmany);
			for (n = howmany[1] - 2; n > 0; n--)
				RemoveEmptySeparator(1, n, howmany);
		}
	}
}

inline void GridView::DragSeparator
(int32 x, int32 y, float cx, float cy, uint32 buttons)
{
	int32 idx[2] = {x, y};
	float coords[2] = {cx, cy};
	DragSeparator(idx, coords, buttons);
}

inline void GridView::AddChild(BView *v, int32 x, int32 y)
{
	AddChild(v, views->Get(x,y));
}

inline bool GridView::RemoveChild(int32 id)
{
	BView *v = (BView *)children.ItemAt(id);

	if (v && BView::RemoveChild(v)) {
		children.RemoveItem(id);
		children.AddItem(NULL, id);
		ChildDetached(v);
		return true;
	}
	else
		return false;
}

inline bool GridView::RemoveChild(int32 x, int32 y)
{
	return RemoveChild(views->Get(x,y));
}

inline bool GridView::RemoveChild(BView *v)
{
	int32 n = children.IndexOf(v);

	if (n >= 0 && BView::RemoveChild(v)) {
		children.RemoveItem(n);
		children.AddItem(NULL, n);
		ChildDetached(v);
		return true;
	}
	else
		return false;
}

inline int32 GridView::Id(int32 x, int32 y) const
{
	return views->Get(x,y);
}

inline int32 GridView::Id(BPoint where) const
{
	int32 idx[2];
	float coords[2];

	coords[0] = where.x;
	coords[1] = where.y;
	GetCoords(coords, idx, 0);

	return views->Get(idx[0], idx[1]);
}

inline BView *GridView::ChildAt(int32 id) const
{
	if (id != -1)
		return (BView *)children.ItemAt(id);
	else
		return NULL;
}

inline BView *GridView::ChildAt(int32 x, int32 y) const
{
	return ChildAt(Id(x,y));
}

inline int32 GridView::CountSeparators(int32 dir) const
{
	if (!dir)
		return separators[0].CountItems();
	else
		return separators[1].CountItems();
}

inline float GridView::SeparatorAt(int32 dir, int32 index) const
{
	float *fp;

	if (dir) dir = 1;
	fp = (float *)separators[dir].ItemAt(index);
	if (fp) return *fp;
	else return 0;
}

#endif /* __GRIDVIEW_H */
